Package: vulcan
Type: Package
Title: VirtUaL ChIP-Seq data Analysis using Networks
Version: 1.35.0
Author: Federico M. Giorgi, Andrew N. Holding, Florian Markowetz
Maintainer: Federico M. Giorgi <federico.giorgi@gmail.com>
Description: Vulcan (VirtUaL ChIP-Seq Analysis through Networks) is a
        package that interrogates gene regulatory networks to infer
        cofactors significantly enriched in a differential binding
        signature coming from ChIP-Seq data. In order to do so, our
        package combines strategies from different BioConductor
        packages: DESeq for data normalization, ChIPpeakAnno and
        DiffBind for annotation and definition of ChIP-Seq genomic
        peaks, csaw to define optimal peak width and viper for applying
        a regulatory network over a differential binding signature.
License: LGPL-3
biocViews: SystemsBiology, NetworkEnrichment, GeneExpression, ChIPSeq
NeedsCompilation: no
Suggests: vulcandata
Depends: R (>= 4.0), ChIPpeakAnno,TxDb.Hsapiens.UCSC.hg19.knownGene,
        zoo, GenomicRanges, S4Vectors, viper, DiffBind, locfit
Imports: wordcloud, csaw, gplots, stats, utils, caTools, graphics,
        DESeq2, Biobase
Encoding: UTF-8
RoxygenNote: 7.1.1
Config/pak/sysreqs: cmake make libbz2-dev libicu-dev libjpeg-dev
        liblzma-dev libpng-dev libuv1-dev libxml2-dev libssl-dev
        xz-utils zlib1g-dev
Repository: https://bioc.r-universe.dev
Date/Publication: 2026-04-28 12:46:32 UTC
RemoteUrl: https://github.com/bioc/vulcan
RemoteRef: HEAD
RemoteSha: c42f6e949d3182a7e7e35e0bf27b67f98174f5bb
Packaged: 2026-05-29 09:34:02 UTC; root
Built: R 4.6.0; ; 2026-05-29 11:18:02 UTC; unix
