# Generated by roxygen2: do not edit by hand

export(average_fragment_length)
export(corr2p)
export(densityauc)
export(dpareto)
export(fisherp)
export(gsea)
export(kmgformat)
export(null_gsea)
export(p2corr)
export(p2z)
export(pareto.fit)
export(plot_gsea)
export(ppareto)
export(rea)
export(slice)
export(stouffer)
export(textplot2)
export(val2col)
export(vulcan)
export(vulcan.annotate)
export(vulcan.import)
export(vulcan.normalize)
export(vulcan.pathways)
export(wstouffer)
export(z2p)

import(zoo)
import(ChIPpeakAnno)
import(locfit)
import(TxDb.Hsapiens.UCSC.hg19.knownGene)
importFrom('GenomicRanges','GRanges')
importFrom('S4Vectors','Rle')
importFrom('gplots','colorpanel')
importFrom('caTools','runmean')
importFrom('csaw','correlateReads')
importFrom('wordcloud','wordlayout')
importFrom('DESeq2','DESeqDataSetFromMatrix','varianceStabilizingTransformation')
importFrom('Biobase','exprs')
importFrom('viper','rowTtest','msviper','msviperAnnot','ttestNull')
importFrom('DiffBind','dba','dba.count')
importFrom('graphics', 'abline', 'grid', 'layout', 'legend', 'lines',
    'par', 'plot', 'points', 'rect', 'text')
importFrom('stats', 'ks.test', 'pchisq', 'pnorm', 'pt', 'qnorm', 'qt',
    'rnorm', 'setNames', 'quantile', 'var',
    'predict', 'smooth.spline')
importFrom('utils', 'read.csv', 'setTxtProgressBar', 'txtProgressBar', 'relist')
